'Filename   SWRMeter_Mega328P_301.bas
'Version    3.01 Arduino NANO
'Date       2017-08-25
'Purpose    SWR and Power meter with
'           - 2 log amps for forward and reverse power measurement
'           - display of SWR & RF forward average or peak envelope power (PEP)
'           - Choose toroid turns of tandem match coupler fitting max. RF power
'           - UHF/SHF couplers with suitable log amps can be used as well
'           - 4 different couplers can be defined with EEPROM data
'           - 2 point calibration at selectable min and max log amp power levels
'           - Forward/reverse power is calculated from straight cal line
'Author     Georg Latzel, DL6GL
'Copyright  (c) Georg Latzel, free for private use under GNU license
'Hardware   Arduino NANO (ATmega328p)
'           Crystal 16 MHz
'           Port  PB1: LCD backlight PWM
'           Ports PB2, PB3, PB4: Keys down, up, OK/set
'           Ports PC0, PC1 (ADC0, ADC1): power Fwd, Rev
'           Ports PC2, PC3 (ADC2, ADC3): binary coupler select
'           Port  PC4: LED_SWR is lit when SWR > 3
'           Port  PC5: LED_Power eff./peak HF power
'           Ports PD2 ... PD7: 2x16 Text LCD
'           ADC AREF: 2.5V Zener diode LM336
'Compiler   BASCOM-AVR 2.0.7.6
'           For readable "camel case" variables mark "Don't change case"
'           in BASCOM Options-Programmer-Environment-Editor
'Edit history
'V1.00      2012-08-04, diode version first release
'V1.10      2013-09-11
'           BASCOM "GETADC" replaced by ADC register handling (faster)
'           Added: Jumper/switch for HF eff/peak power display
'           Added: LED_Power to show HF eff/peak power setting
'           Added: LED_SWR: SWR > 3 alert
'           SWR bridge data set and included via SWR_Bridge_Data.bas
'           Prepared for future setup option, not implemented yet
'V2.00      2013-11-02
'           Added: Coupler data setup, 2 couplers may be selected
'V2.01      2014-03-05
'           Repetitive ADC runs increased from 3 to 7
'           New: Voltage measurement calibration factors for the 2 couplers/units
'           EEPROM storage for the 2 couplers changed
'V2.02      2015-10-16
'           No bar displayed for SWR=1
'V3.00      2017-03-03
'           2 log amps, e.g. AD8307, are used instead of diodes in the coupler
'           4 couplers may be configured
'           PEP measurement in forward channel
'           Arduino NANO is used (ATmega328P)
'           Flash used: 18,200 bytes (55%)
'V3.01      2017-08-25
'           FWD lower detection limit is used to indicate TX is on/off
'           SWR display suppressed if FWD power < lower detection limit (TX off)
'           REV lower detection limit = FWD lower detection limit
'                                       if it is below lower calibration point
'                                     = lower calibration point
'                                       if it is above lower calibration point
'           Flash used: 18,406 bytes (56%)
'==============================================================================
'2 main menu levels, selected by Mode key
'Menu level 1: normal operation, display power and SWR. Keys are not used.
'              If data of the selected coupler are not found, menu level 2 entered
'Menu level 2: Setup data for the selected coupler
'              Setup is activated if PC0 is connected to GND, keys are used.
'submenus:
' (2, 1)       LCD backlight brightness
' (2, 2)       ADC runs for mean values
' (2, 3)       Coupling factor
' (2, 4)       Log amp attenuators
' (2, 5)       Calibration frequency
' (2, 6)       Linear 2 point calibration, lower dBm level
' (2, 7)       Linear 2 point calibration, upper dBm level
' (2, 8)       Lower detection limit
' (2, 9)       Upper detection limit
' (2,10)       PEP peak sample & hold time
' (2,11)       Calibrate FWD low  power level
' (2,12)       Calibrate REV low  power level
' (2,13)       Calibrate FWD high power level
' (2,14)       Calibrate REV high power level
'==============================================================================
'FUSE-Bits (AVR Studio 4.0-Settings, [ ]=no check, [x]=check)
'In ATMEGA-Data sheets: Bit = 1 -> not set = no check
'                       Bit = 0 -> set     = check
'BODLEVEL   Brown-out detection at VCC=4.3V
'RSTDISBL   [ ]
'DWEN       [ ]
'SPIEN      [x] Do not change!
'WDTON      [ ]
'EESAVE     [x]
'BOOTSZ     Boot Flash size=256 words (minimal boot size)
'BOOTRST    [ ]
'CKDIV8     [ ]
'CKOUT      [ ]
'SUT_CKSEL  Ext. Crystal Osc. 8.0- MHz, Start-up time 16k + 14ms
'Extended:  0xFC
'High:      0xD7
'Low:       0xFF

$regfile = "m328pdef.dat"                         'ATMega328P
$crystal = 16000000                               '16 MHz xtal
$hwstack = 64
$swstack = 64
$framesize = 64

'Configure in/out on Port B ---------------------------------------------------
'Not used                     D8  PB0(IC01)
'PWM LCD backlight            D9  PB1(OC1A)
'Down key                     D10 PB2
'Up   key                     D11 PB3(MOSI)
'OK   key                     D12 PB4(MISO)
'Up and Down keys with autorepeat to set data
'OK key pressed short: next setup submenu, current data are not changed
'OK key pressed long : store current setting to EEPROM
DDRB = &B00000010                                 'All input except PB1
PortB = &B00011101                                'Pullup PB0, PB2...PB5
Const Key_Down = 2                                'Key Down        @ PinB.2
Const Key_Up = 3                                  'Key Up          @ PINB.3
Const Key_OK = 4                                  'Key OK          @ PinB.4
Key_port Alias PinB                               'Input-Port Keys
Dim bytKey_Pressed As Byte                        '>0 if any key  is pressed
Dim bytKey_Up As Byte                             '=1 if key up   is pressed
Dim bytKey_Down As Byte                           '=1 if key down is pressed
Dim bytKey_OK As Byte                             'Key OK  pressed: 1(short), 2(long)
Dim bytKey_mask As Byte                           'Mask for short/long keys
bytKey_Mask = &B00011100                          'Set 1 to enable short/long
'                                                  Keys #2 & 3 = Up&Down autorepeat
'                                                  Keys #4 = OK short/long

'Configure ADC, Coupler select & LED on Port C --------------------------------
'ADC0                         A0  PC0             'Forward power
'ADC1                         A1  PC1             'Reverse power
'Coupler select jumper 1      A2  PC2
'Coupler select jumper 2      A3  PC3
'LED SWR SWR > 3              A4  PC4             'lit: SWR > 3
'LED Power display            A5  PC5             'lit: avg. power, PEP else
'Not used                     A6
'Not used                     A7
DDRC = &B00110000                                 'All input except PC4/PC5
PORTC = &B00001100                                'Pullup PC2/PC3
Jmp_Coup1 Alias PINC.2                            'Coupler selection bit 1
Jmp_Coup2 Alias PINC.3                            'Coupler selection bit 2
'Jumper coupler selection     A2  PC2             '  out  in  out  in
'Jumper coupler selection     A3  PC3             '  out  out in   in
'Selected coupler                                 '  1    2   3    4
LED_SWR Alias PortC.4                             'Output LED_SWR @ PortC.4
LED_Power Alias PortC.5                           'Output LED_Power @ PortC.5

'Configure LCD & PWM LCD backlight on Port D ----------------------------------
'LCD RS                       D7  PD7
'LCD E                        D6  PD6
'LCD D4                       D5  PD5
'LCD D5                       D4  PD4
'LCD D6                       D3  PD3
'LCD D7                       D2  PD2
'Not used                     TX1 PD1
'Not used                     RX0 PD0
Config Lcd = 16 * 2
Config Lcdpin = Pin , Db7 = PortD.2 , Db6 = PortD.3 , Db5 = PortD.4
Config Lcdpin = Pin , Db4 = PortD.5 , E = PortD.6 , Rs = PortD.7
Config Lcdbus = 4

Const bytPWMmax = 240                             'Full LCD backlight brightness
Const bytChar = 16                                'Characters per LCD line
Const bytBarLen = 12                              'Bar length (characters)

'Define display bars ----------------------------------------------------------
Deflcdchar 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0      'not used
Deflcdchar 1 , 16 , 16 , 16 , 16 , 16 , 16 , 16 , 16       'chr(1): 1 bar
Deflcdchar 2 , 24 , 24 , 24 , 24 , 24 , 24 , 24 , 24       'chr(2): 2 bars
Deflcdchar 3 , 28 , 28 , 28 , 28 , 28 , 28 , 28 , 28       'chr(3): 3 bars
Deflcdchar 4 , 30 , 30 , 30 , 30 , 30 , 30 , 30 , 30       'chr(4): 4 bars
Deflcdchar 5 , 31 , 31 , 31 , 31 , 31 , 31 , 31 , 31       'chr(5): 5 bars (full)
Cls

'Variables program control ----------------------------------------------------
Dim bitSetup As Bit                               '=0: normal operation, =1: setup
Dim bytSetup As Byte                              '=0: enter setup, =2: change value
Dim bytMenu As Byte                               'Setup submenu number
Dim bytCursorFirst As Byte                        'First LCD cursor position

'Common Variables, multiply used ----------------------------------------------
Dim bytTmp0 As Byte
Dim bytTmp1 As Byte
Dim bytTmp2 As Byte
Dim wrdTmp0 As Word
Dim wrdTmp1 As Word
Dim wrdTmp2 As Word
Dim intTmp0 As Integer
Dim intTmp1 As Integer
Dim intTmp2 As Integer
Dim sngTmp0 As Single
Dim sngTmp1 As Single
Dim sngTmp2 As Single
Dim strDis1 As String * 16                        'Text to be dispayed on LCD
Dim bytDis1(16) As Byte At strDis1 overlay
Dim strDis2 As String * 16
Dim strTmp16 As String * 16
Dim bytTmp16(16) As Byte At strTmp16 Overlay

'Variables Keys ---------------------------------------------------------------
Dim bytIwr0 As Byte
Dim bytKey_ct0 As Byte
Dim bytKey_ct1 As Byte
Dim bytKey_state As Byte
Dim bytKey_press As Byte
Dim bytKey_save As Byte
Dim bytTimer0_keys As Word                        'Timer0 counter for keys
Const bytTimer0_long = 80                         'Timer0 counts for long press

'ADC Variables ----------------------------------------------------------------
Dim bytADMUX As Byte                              'ADC MUX channel
Dim bytADCOvl As Byte                             '=i: ADC overflow in channel i, =0: else
Dim wrdADC As Word                                'ADC data register content
Dim bytADCLo As Byte At wrdADC Overlay            'ADC data register lo byte
Dim bytADCHi As Byte At wrdADC + 1 Overlay        'ADC data register hi byte
Dim bitADCdone As Bit                             '1= all ADC channels complete
Dim bitADCMean As Bit                             '=1: Power accumulation done
Dim bytADCRun As Byte                             'Current ADC run for mean value
Dim bytADChannel As Byte                          '0=Forward, 1=Reverse
Dim bytADCValIdx As Byte                          'Current ADC value index
Dim bytPWRerr As Byte
Dim wrdADCVal(2) As Word                          'ADC counts, 1=Forward, 2=Reverse
Dim wrdADCMean(2) As Word                         'ADC mean counts, 1=Forward, 2=Reverse
Const bytADCUsed = 1                              'Number of used ADC channels - 1

'Coupler data -----------------------------------------------------------------
Dim bytCoupler As Byte                            'Coupler number 1...4
Dim bytCouplerData As Byte                        '=0: coupler initialized, =1: calibrated
Dim bytLCDPWM As Byte                             'PWM value for LCD backlight
Dim bytADCRep As Byte                             'Repetitive ADC runs for mean value
Dim bytPEPhold As Byte                            'PEP peak sample & hold time (sec)
Dim bytData As Byte                               'Handled in Sub SetValue
Dim bytDataMin As Byte                            'Handled in Sub SetValue
Dim bytDataMax As Byte                            'Handled in Sub SetValue
Dim bytDataInc As Byte                            'Handled in Sub SetValue
Dim bytDataDec As Byte                            'Handled in Sub SetValue
Dim wrdData As Word                               'Handled in Sub SetValue
Dim wrdDataMin As Word                            'Handled in Sub SetValue
Dim wrdDataMax As Word                            'Handled in Sub SetValue
Dim wrdCouplerAddr As Word                        'Coupler EEPROM start address
Dim wrdCoupFac As Word                            'Coupling factor (dB x 10)
Dim wrdAmpAtt As Word                             'Log amp attenuator (dB x 10)
Dim wrdCalFreq As Word                            'Coupler calibration frequency
Dim wrdCalFwdADCLo As Word                        'Forward ADC counts at low  cal. point
Dim wrdCalRevADCLo As Word                        'Reverse ADC counts at low  cal. point
Dim wrdCalFwdADCHi As Word                        'Forward ADC counts at high cal. point
Dim wrdCalRevADCHi As Word                        'Reverse ADC counts at high cal. point
Dim intCaldBmLo As Integer                        'Low  dBm calibration point (dBm x 10)
Dim intCaldBmHi As Integer                        'High dBm calibration point (dBm x 10)
Dim intMindBm As Integer                          'Lower detection limit (dBm x 10)
Dim intMaxdBm As Integer                          'Upper detection limit (dBm x 10)
Dim intData As Integer                            'Handled in Sub SetValue
Dim intDataMin As Integer                         'Handled in Sub SetValue
Dim intDataMax As Integer                         'Handled in Sub SetValue
Dim sngMinFWDdBm As Single                        'Lower FWD detection limit (dBm)
Dim sngMinREVdBm As Single                        'Lower REV detection limit (dBm)
Dim sngMaxdBm As Single                           'Upper detection limit (dBm)
Dim sngCoupAtt As Single                          'Total coupler attenuation (dB)
Dim sngADCRep As Single
Dim strUnit As String * 3                         'Handled in Sub SetValue

'SWR & power Variables --------------------------------------------------------
Dim bitShowAvg As Bit                             '=0: Show PEP, =1: show avg. Power
Dim bitTXOFF As Bit                               '=1: TX is off, =0 TX is on
Dim wrdTimer0_PEP As Word                         'Timer0 counter for PEP peak hold
Dim wrdTimer0_PEPhold As Word                     'Timer0 counts for PEP peak hold
Dim wrdADCPEP As Word                             'Current max ADC PEP value
Dim wrdPEP As Word                                'Final max ADC PEP value
Dim sngFwddBm As Single                           'Forward power (dBm)
Dim sngRevdBm As Single                           'Reverse power (dBm)
Dim sngFwdPwr As Single                           'Forward power (W)
Dim sngRevPwr As Single                           'Reverse power (W)
Dim sngFwdGrd As Single                           'Cal line gradient forward
Dim sngFwdOrd As Single                           'Cal line ordinate forward
Dim sngRevGrd As Single                           'Cal line gradient reverse
Dim sngRevOrd As Single                           'Cal line ordinate Reverse
Dim sngSWR As Single                              'SWR
Dim sngSWRBarCal As Single                        'SWR bar scale factor
Const sngSWRBarMax = 3                            'Max SWR for bargraph
Const sngSWRMax = 3                               'Max. SWR: LED_SWR is lit

'EEPROM data ------------------------------------------------------------------
Dim wrdAddr As Word
Dim wrdEEPROM As Word                             'Word variable to/from EEPROM
Dim bytWrdEEP(2) As Byte At wrdEEPROM Overlay
Dim intEEPROM As Integer                          'Integer variable to/from EEPROM
Dim bytIntEEP(2) As Byte At intEEPROM Overlay

'Declare subs & functions -----------------------------------------------------
'Display subs
Declare Sub ClearLCDLine(byVal bytRow As Byte , byVal bytChars As Byte)
Declare Sub DisplayString(byVal bytPosLast As Byte)
'Key subs
Declare Function KeyPressedLong(byVal bytKey_Long As Byte) As Byte
Declare Function KeyAutoRepeat(byVal bytKey_Rep As Byte) As Byte
'EEPROM & EEPROM setup subs
Declare Sub CheckEEPROM
Declare Sub EEPROMReadWrd(byRef wrdVal As Word)
Declare Sub EEPROMWriteWrd(byRef wrdVal As Word)
Declare Sub EEPROMReadInt(byRef intVal As Integer)
Declare Sub EEPROMWriteInt(byRef intVal As Integer)
Declare Sub SetValue(byVal bytRow As Byte , byVal bytType As Byte , byVal wrdScale As Word)
Declare Sub LogAmpCal(byVal bytChannel As Byte , byVal bytAddrdBm As Byte , byVal bytAddrADC As Byte)
Declare Function CouplerDataComplete(byVal bytCoupNum As Byte) As Byte
'Power & SWR subs
Declare Function dBm2Watt(byVal sngdBm As Single) As Single
Declare Sub GetADCData
Declare Sub TwoPointCal(byVal bytOpt As Byte)
Declare Sub ShowPower(byVal bytRowPwr As Byte)
Declare Sub ShowSWR(byVal bytRowSWR As Byte)

'Configure Timer0-Interrupt (Timer0 = 8bit, 2^8=256 counts) -------------------
'Possible Prescales: 8, 64, 256, 1024, here 1024
'For 16.000 MHz xtal:
'Timer0 overflow counts are reduced by PresetTimer0 = 56 to 256-56=200
'Overflow time: 200 * 1024 / 16,000,000 = 12.8 msec (78.125 Hz)
Config Timer0 = Timer , Prescale = 1024
Const Presettimer0 = 56
On Timer0 Timer0_isr                              'On Overflow go to Timer0_isr

'Configure Timer1 for LCD backlight pulse width modulation (PWM) --------------
'Timer1 generates a triangle with 2^8-1=255 steps going up and downwards resp.
'PWM frequency=Clock/(Prescaler*(2^(bit resolution+1)))
'Clock=16MHz, prescaler=256, bit resolution=8 -> PWM frequency=122Hz
'bytLCDPWM = 0...255 gives a square signal at PD.7(OC2)...
'...with constant PWM frequency and varying duty cycle
'PWM output = PB.1(OC1A)
Config Timer1 = PWM , PWM = 8 , Prescale = 256 , Compare A Pwm = Clear Up

'Configure ADC in single running mode, Reference = AVCC (ATmega328P) ----------
'Reference = external 2.5V
'ADC data right adjusted
'Start with ADC channel 0
'Enable ADC
'ADC clock prescaler 128 -> ADC clock 125kHz (xtal 16MHz)
'ADC not yet started
'Conversion completion interrupt activated
bytADMUX = &B00000000
Admux = bytADMUX                                  'Reference = ext AREF, ADLAR=0, ADC0
'                                                  ADC Multiplexer Selection Register
'                                                  Bit 7 , 6 : REFS1 , REFS0
'                                                  00: external reference @ AREF
'                                                  01: Reserved
'                                                  11: Internal 1.10V Reference
'                                                  Bit 5, ADLAR=0: data right adjusted
'                                                              =1: data left adjusted
'                                                  Bit 4 reserved
'                                                  Bit 3-0, MUX3...MUX0
'                                                  =0000: ADC0 is input
'                                                  =0001: ADC1 is input
'                                                  =0010: ADC2 is input
'                                                  =0011: ADC3 is input
'                                                  =0100: ADC4 is input
'                                                  =0101: ADC5 is input
'                                                  =0110: ADC6 is input
'                                                  =0111: ADC7 is input
ADCSRA = &B10001111                               'Enable ADC, single mode, div. factor 128
'                                                  ADC Control & Status Register A
'                                                  Bit 7, ADEN = 1: enable ADC
'                                                  Bit 6, ADSC = 1: Start ADC
'                                                  Bit 5, ADATE = 1: Auto trigger enabled
'                                                  Bit 4, ADIF = 1 when conversion completed
'                                                  Bit 3, ADIE = 1: Conversion completion interrupt activated
'                                                  Bit 2-0, ADPS2...ADPS0, ADC prescaler
'                                                  =000: Div. factor 2
'                                                  =001: Div. factor 2
'                                                  =010: Div. factor 4
'                                                  =011: Div. factor 8
'                                                  =100: Div. factor 16
'                                                  =101: Div. factor 32
'                                                  =110: Div. factor 64
'                                                  =111: Div. factor 128
ADCSRB = &B00000000                               'ADC Control & Status Register B
'                                                  Bit 7, 5...3: Reserved, Set 00000
'                                                  Bit 2-0, ADTS2...ADTS0, Auto trigger source
'                                                  =000: Free running mode
'                                                  =001: Analog comparator
'                                                  =010: Ext. Interrupt Request 0
'                                                  =011: Timer/Counter0 Compare match
'                                                  =100: Timer/Counter0 Overflow
'                                                  =101: Timer/Counter1 Compare match
'                                                  =110: Timer/Counter1 Overflow
'                                                  =111: Timer/Counter1 Capture event
DIDR0 = &B00110000                                'Digital Input Disable Register 0
'                                                  Any bit = 0: corresponding ADC input enabled
'                                                  Any bit = 1: corresponding ADC input disabled
On ADC ADC_isr                                    'ADC conversion complete interrupt

'Initialize Key Data ----------------------------------------------------------
bytKey_ct0 = 0
bytKey_ct1 = 0
Decr bytKey_ct0
bytKey_ct1 = bytKey_ct0
bytKey_state = 0
bytKey_press = 0

'Start timers -----------------------------------------------------------------
Enable Timer0                                     'Timer0 for keys & PEP timing
Enable Interrupts                                 'enable all Interrupts
Timer0 = Presettimer0                             'initialize Timer0
bytTimer0_keys = 0

'OK key is pressed while power on: Enter Setup --------------------------------
If Key_port.Key_OK = 0 Then                       'OK key is pressed
   bitSetup = 1                                   'Enter setup
Else
   bitSetup = 0                                   'Normal operation
End If

'Splash screen ----------------------------------------------------------------
OCR1A = bytPWMmax                                 'LCD backlight full brightness
Cls
Cursor Off
Locate 1 , 1
LCD "SWR-Meter 3.01"
Locate 2 , 1
LCD "DL6GL 2017-08-25"
Wait 1
Cls

'Get coupler selection from Jmp_Coup1 & Jmp_Coup2 -----------------------------
bytTmp0 = 0
bytTmp0.0 = Jmp_Coup1
bytTmp0.1 = Jmp_Coup2
bytTmp1 = &B00000011                              'Inverting mask
bytTmp0 = bytTmp0 XOR bytTmp1                     'Inverted bit pattern
bytCoupler = bytTmp0 + 1                          'Coupler number 1...4

'Initialize data --------------------------------------------------------------
Restart:                                          'Restart after setup

Call CheckEEPROM                                  'Read/initialize EEPROM data
sngADCRep = bytADCRep
wrdADCPEP = 0                                     'ADC max  counts forward
wrdADCMean(1) = 0                                 'ADC mean counts forward
wrdADCMean(2) = 0                                 'ADC mean counts reverse
wrdTimer0_PEP = 0                                 'Timer0 counter for PEP peak hold
sngTmp0 = bytPEPhold                              'PEP hold time (sec x 10)
sngTmp0 = 0.1 * sngTmp0                           'PEP hold time (sec)
sngTmp0 = sngTmp0 / .0128                         'PEP hold time Timer0 cycles
wrdTimer0_PEPhold = sngTmp0                       'Timer0 counts for PEP peak hold
bytADCRun = 0
bytADChannel = 0                                  'First ADC channel ADC0
bitADCdone = 0
Cls
OCR1A = bytLCDPWM                                 'Set PWM value on OC1A (PB.1)
bitShowAvg = 0                                    'Show peak envelope power
LED_Power = bitShowAvg

If bitSetup = 0 Then
   '===========================================================================
   'Normal operation, SWR-/Powermeter
   '===========================================================================
   If bytCouplerData = 1 Then                     'Coupler data available
      '2 point calibration line for selected coupler
      Call TwoPointCal(1)                         'Forward
      Call TwoPointCal(2)                         'Reverse
      sngTmp1 = wrdCoupFac                        'Coupling factor (dB x 10)
      sngTmp1 = 0.1 * sngTmp1                     'Coupling factor (dB)
      sngTmp2 = wrdAmpAtt                         'Log amp attenuator (dB x 10)
      sngTmp2 = 0.1 * sngTmp2                     'Log amp attenuator (dB)
      sngCoupAtt = sngTmp1 + sngTmp2              'Total coupler attenuation (dB)
      'Set lower detection limits                  FWD:  set in setup #8
      '                                                  detection TX on/off
      '                                            REV:  set in setup #8 if < low cal. point
      '                                                  = low cal. point if > setup #8
      sngTmp1 = intMindBm                         'Lower detection limit (dBm x 10)
      sngTmp1 = 0.1 * sngTmp1                     'Lower detection limit (dBm)
      sngMinFWDdBm = sngTmp1                      'Lower FWD detection limit (dBm)
      If intMindBm < intCaldBmLo Then             'Setup #8 low det. limit below low cal. point
         sngMinREVdBm = sngTmp1                   'REV low det. limit from setup #8
      Else
         sngTmp2 = intCaldBmLo                    'Low  dBm calibration point(dBm x 10)
         sngTmp2 = 0.1 * sngTmp2                  'Low  dBm calibration point(dBm)
         sngMinREVdBm = sngTmp2                   'REV low det. limit = low cal. point
      End If

      sngMaxdBm = intMaxdBm                       'Upper detection limit (dBm x 10)
      sngMaxdBm = 0.1 * sngMaxdBm                 'Upper detection limit (dBm)
      sngSWRBarCal = bytBarLen * 5                'SWR bar dots (5 per character)
      sngTmp0 = sngSWRBarMax - 1
      sngSWRBarCal = sngSWRBarCal / sngTmp0       'SWR bar calibration, SWR=1 suppressed
      ADCSRA.ADSC = 1                             'Start ADC

      Do

         If bitADCdone = 1 Then                   'ADC accumulation complete
            Call GetADCData                       'Handle ADC values PEP & Avg.
            If bitADCMean = 1 Then                'ADC mean data available
               'Check keys , change power display PEP / Avg
               bytKey_Pressed = bytKey_press AND bytKey_mask       'Mask out keys
               If bytKey_Pressed > 0 Then         'Any key was pressed
                  'Check key down
                  If bytKey_press.Key_down = 1 Then       'Key down was pressed
                     Toggle bitShowAvg            'Change PEP/AVG
                     Toggle LED_Power
                     bytKey_press = 0             'Reset key pressed flag
                  End If
               End If
               Call ShowPower(1)                  'Show power in row 1
               If bytPWRerr = 0 Then              'FWD & REV are OK
                  Call ShowSWR(2)                 'Show SWR in row 2
               Else
                  Call ClearLCDLine(2 , bytChar)
               End If
               bytADCRun = 0                      'Initialize next ADC run
               wrdADCMean(1) = 0
               wrdADCMean(2) = 0
            End If
         End If

      Loop

   Else                                           'coupler is not calibrated
      Call ClearLCDLine(1 , bytChar)
      LCD "Unit" ; Str(bytCoupler) ; " not calib."
      Call ClearLCDLine(2 , bytChar)
      LCD "Setup data pls."
      Wait 2
      bitSetup = 1
      GoTo Restart

   End If

Else                                              'Setup jumper in: setup
   '===========================================================================
   'Setup mode
   '===========================================================================

   bytMenu = 21                                   'Setup start menu: ADC ref voltage
   bytSetup = 0                                   'Setup init
   wrdCouplerAddr = bytCoupler - 1
   wrdCouplerAddr = wrdCouplerAddr * 30
   wrdCouplerAddr = wrdCouplerAddr + 10           'Coupler EEPROM Start address

   Do
      'Check keys, any key pressed? -------------------------------------------
      'bytKey_press returned from Timer0_isr
      'Key_Down:  #2
      'Key_Up:    #3
      'Key_OK:   #4
      'Just pressed detected in Timer0_isr: pressed status flag bit = 1 bytKey_press.x
      '2 key flags are issued from Timer0_isr:
      '-1  bytKey_press.x = 1: key x changed from open to pressed,
      '    Used for normal key pressed action
      '    After evaluating bytKey_press.x flag is reset just below.
      '    Short/long pressed status flag is reset in function KeyPressedLong
      '-2  bytKey_state.x = 1: while key is pressed, = 0: Key is released
      '    bytKey_state.x flag is used for autorepeat
      'Key action is stored in the variables:
      'bytKey_Up     = 0: Key Up was not pressed
      '              = 1: Key Up was pressed short
      '              = 2: Key Up was pressed long, auto repeat
      'bytKey_Down   = 0: Key Down was not pressed
      '              = 1: Key Down was pressed short
      '              = 2: Key Down was pressed long, auto repeat
      'bytKey_OK     = 0: Key OK  was not pressed
      '              = 1: Key OK  was pressed short
      '              = 2: Key OK  was pressed long

      bytKey_Pressed = bytKey_press AND bytKey_mask       'Mask out keys
      If bytKey_Pressed > 0 Then                  'Any key was pressed
         'Check short/long key OK
         If bytKey_press.Key_OK = 1 Then          'Key OK was pressed
            bytKey_OK = KeyPressedLong(Key_OK)    'Pressed short or long
         Else
            bytKey_OK = 0                         'Key OK not pressed
            bytKey_press.Key_OK = 0               'Reset Key OK pressed flag
         End If
         'Check auto repeat keys Up & Down
         If bytKey_press.Key_Up = 1 Then          'Key Up was pressed
            bytkey_Up = KeyAutoRepeat(Key_Up)     'Pressed short or auto repeat
         Else
            bytKey_state.Key_Up = 0               'Reset Key Up status  flag
         End If
         If bytKey_press.Key_Down = 1 Then        'Key Down was pressed
            bytkey_Down = KeyAutoRepeat(Key_Down) 'Pressed short or auto repeat
         Else
            bytKey_state.Key_Down = 0             'Reset Key Down status  flag
         End If
      Else                                        'No key was pressed
         bytKey_Up = 0                            'Reset all key flags
         bytKey_Down = 0
         bytKey_OK = 0
         bytKey_Pressed = 0
      End If

      Select Case bytMenu
         Case 21                                  'Setup LCD backlight PWM
            wrdAddr = 2
            If bytSetup = 0 Then                  'Prepare data
               bytSetup = 1
               Locate 1 , 1
               bytTmp0 = bytMenu - 20
               LCD "Setup " ; bytTmp0 ; "/10      "
               ReadEEPROM bytLCDPWM , wrdAddr     'PWM value for LCD backlight
               strUnit = " "
               bytData = bytLCDPWM
               bytDataMin = 0                     'Min PWM value
               bytDataMax = bytPWMmax             'Max PWM value
               bytDataInc = 10                    'Increment factor
               bytDataDec = 10                    'Decrement factor
               Call ClearLCDLine(2 , bytChar)
               LCD "LCD PWM"
               bytCursorFirst = 9
               Locate 2 , bytCursorFirst
            Else                                  'Customize data
               Call SetValue(2 , 1 , 1)           'Row 2, Byte, Scale 1
            End If

         Case 22                                  'Setup ADC runs for mean value
            wrdAddr = 3
            If bytSetup = 0 Then                  'Prepare data
               bytSetup = 1
               Locate 1 , 1
               bytTmp0 = bytMenu - 20
               LCD "Setup " ; bytTmp0 ; "/10      "
               ReadEEPROM bytADCRep , wrdAddr     'Repetitive ADC runs for mean value
               strUnit = " "
               bytData = bytADCRep
               bytDataMin = 1                     'Min repetition value
               bytDataMax = 50                    'Max repetition value
               bytDataInc = 10                    'Increment factor
               bytDataDec = 1                     'Decrement factor
               Call ClearLCDLine(2 , bytChar)
               LCD "ADC Rep"
               bytCursorFirst = 9
               Locate 2 , bytCursorFirst
            Else                                  'Customize data
               Call SetValue(2 , 1 , 1)           'Row 2, Byte, Scale 1
            End If

         Case 23                                  'Setup coupling factor
            wrdAddr = wrdCouplerAddr + 1
            If bytSetup = 0 Then                  'Prepare data
               bytSetup = 1
               Locate 1 , 1
               bytTmp0 = bytMenu - 20
               LCD "Setup " ; bytTmp0 ; "/10 Unit" ; bytCoupler
               Call EEPROMReadWrd(wrdCoupFac)
               strUnit = "dB"
               wrdData = wrdCoupFac
               wrdDataMin = 100                   'Min coupling factor (dB x 10)
               wrdDataMax = 400                   'Max coupling factor (dB x 10)
               bytDataInc = 10                    'Increment factor
               bytDataDec = 1                     'Decrement factor
               Call ClearLCDLine(2 , bytChar)
               LCD "Cp-Fac"
               bytCursorFirst = 8
               Locate 2 , bytCursorFirst
            Else                                  'Customize data
               Call SetValue(2 , 2 , 10)          'Row 2, Word, Scale 10
            End If

         Case 24                                  'Setup log amp attenuator
            wrdAddr = wrdCouplerAddr + 3
            If bytSetup = 0 Then                  'Prepare data
               bytSetup = 1
               Locate 1 , 1
               bytTmp0 = bytMenu - 20
               LCD "Setup " ; bytTmp0 ; "/10 Unit" ; bytCoupler
               Call EEPROMReadWrd(wrdAmpAtt)
               strUnit = "dB"
               wrdData = wrdAmpAtt
               wrdDataMin = 0                     'Min attenuator (dB x 10)
               wrdDataMax = 400                   'Max attenuator (dB x 10)
               bytDataInc = 10                    'Increment factor
               bytDataDec = 1                     'Decrement factor
               Call ClearLCDLine(2 , bytChar)
               LCD "Amp att"
               bytCursorFirst = 9
               Locate 2 , bytCursorFirst
            Else                                  'Customize data
               Call SetValue(2 , 2 , 10)          'Row 2, Word, Scale 10
            End If

         Case 25                                  'Setup calibration frequency
            wrdAddr = wrdCouplerAddr + 5
            If bytSetup = 0 Then                  'Prepare data
               bytSetup = 1
               Locate 1 , 1
               bytTmp0 = bytMenu - 20
               LCD "Setup " ; bytTmp0 ; "/10 Unit" ; bytCoupler
               Call EEPROMReadWrd(wrdCalFreq)
               strUnit = "MHz"
               wrdData = wrdCalFreq
               wrdDataMin = 1                     'Min cal frequency (MHz)
               wrdDataMax = 3000                  'Max cal frequency (MHz)
               bytDataInc = 10                    'Increment factor
               bytDataDec = 1                     'Decrement factor
               Call ClearLCDLine(2 , bytChar)
               LCD "CalFreq"
               bytCursorFirst = 9
               Locate 2 , bytCursorFirst
            Else                                  'Customize data
               Call SetValue(2 , 2 , 1)           'Row 2, Word, Scale 10
            End If

         Case 26                                  'Setup low Log amp dBm cal. point
            wrdAddr = wrdCouplerAddr + 7
            If bytSetup = 0 Then                  'Prepare data
               bytSetup = 1
               Locate 1 , 1
               bytTmp0 = bytMenu - 20
               LCD "Setup " ; bytTmp0 ; "/10 Unit" ; bytCoupler
               Call EEPROMReadInt(intCaldBmLo)
               strUnit = "dBm"
               intData = intCaldBmLo
               intDataMin = -800                  'Min dBm x 10
               intDataMax = -200                  'Max dBm x 10
               bytDataInc = 10                    'Increment factor
               bytDataDec = 1                     'Decrement factor
               Call ClearLCDLine(2 , bytChar)
               LCD "CalLo"
               bytCursorFirst = 7
               Locate 2 , bytCursorFirst
            Else                                  'Customize data
               Call SetValue(2 , 3 , 10)          'Row 2, Int, Scale 10
            End If

         Case 27                                  'Setup high log amp dBm cal. point
            wrdAddr = wrdCouplerAddr + 9
            If bytSetup = 0 Then                  'Prepare data
               bytSetup = 1
               Locate 1 , 1
               bytTmp0 = bytMenu - 20
               LCD "Setup " ; bytTmp0 ; "/10 Unit" ; bytCoupler
               Call EEPROMReadInt(intCaldBmHi)
               strUnit = "dBm"
               intData = intCaldBmHi
               intDataMin = -200                  'Min dBm x 10
               intDataMax = 300                   'Max dBm x 10
               bytDataInc = 10                    'Increment factor
               bytDataDec = 1                     'Decrement factor
               Call ClearLCDLine(2 , bytChar)
               LCD "CalHi"
               bytCursorFirst = 7
               Locate 2 , bytCursorFirst
            Else                                  'Customize data
               Call SetValue(2 , 3 , 10)          'Row 2, Int, Scale 10
            End If

         Case 28                                  'Setup low log amp detection limit
            wrdAddr = wrdCouplerAddr + 19
            If bytSetup = 0 Then                  'Prepare data
               bytSetup = 1
               Locate 1 , 1
               bytTmp0 = bytMenu - 20
               LCD "Setup " ; bytTmp0 ; "/10 Unit" ; bytCoupler
               Call EEPROMReadInt(intMinDBm)
               strUnit = "dBm"
               intData = intMinDBm
               intDataMin = -800                  'Min dBm x 10
               intDataMax = -200                  'Max dBm x 10
               bytDataInc = 10                    'Increment factor
               bytDataDec = 1                     'Decrement factor
               Call ClearLCDLine(2 , bytChar)
               LCD "LimLo"
               bytCursorFirst = 7
               Locate 2 , bytCursorFirst
            Else                                  'Customize data
               Call SetValue(2 , 3 , 10)          'Row 2, Int, Scale 10
            End If

         Case 29                                  'Setup high log amp detection limit
            wrdAddr = wrdCouplerAddr + 21
            If bytSetup = 0 Then                  'Prepare data
               bytSetup = 1
               Locate 1 , 1
               bytTmp0 = bytMenu - 20
               LCD "Setup " ; bytTmp0 ; "/10 Unit" ; bytCoupler
               Call EEPROMReadInt(intMaxDBm)
               strUnit = "dBm"
               intData = intMaxDBm
               intDataMin = -100                  'Min dBm x 10
               intDataMax = 300                   'Max dBm x 10
               bytDataInc = 10                    'Increment factor
               bytDataDec = 1                     'Decrement factor
               Call ClearLCDLine(2 , bytChar)
               LCD "LimHi"
               bytCursorFirst = 7
               Locate 2 , bytCursorFirst
            Else                                  'Customize data
               Call SetValue(2 , 3 , 10)          'Row 2, Int, Scale 10
            End If

         Case 30                                  'Setup PEP sample&hold time
            wrdAddr = wrdCouplerAddr + 23
            If bytSetup = 0 Then                  'Prepare data
               bytSetup = 1
               Locate 1 , 1
               bytTmp0 = bytMenu - 20
               LCD "Setup" ; bytTmp0 ; "/10 Unit" ; bytCoupler
               ReadEEPROM bytPEPhold , wrdAddr
               strUnit = "sec"
               bytData = bytPEPhold
               bytDataMin = 1                     'Min time (1/10 sec)
               bytDataMax = 99                    'Max time (1/10 sec)
               bytDataInc = 10                    'Increment factor
               bytDataDec = 1                     'Decrement factor
               Call ClearLCDLine(2 , bytChar)
               LCD "PEP hold"
               bytCursorFirst = 9
               Locate 2 , bytCursorFirst
            Else                                  'Customize data
               Call SetValue(2 , 1 , 10)          'Row 2, Byte, Scale 10
            End If

         Case 31                                  'Calibrate log amp?
            'If calibration data are already stored, ask for renewal
            If bytCouplerData = 1 Then
               Locate 1 , 1
               LCD "Replace calib.? "
               Locate 2 , 1
               LCD "Y(OKlong), N(Up)"
               If bytKey_OK = 2 Then              'Set pressed long: replace cal
                  bytMenu = 32                    'Calibrate Log amp
               End If
               If bytKey_Up > 0 Then              'Up pressed, no cal replacement
                  bytMenu = 36                    'Setup finished
               End If
            Else
               bytMenu = 32                       'Calibrate Log amp
            End If

         Case 32                                  'Calibrate FWD low power level
            'Coupler in normal operation: Tx @ TX in, Dummy load @ Ant out
            'FWD port 1 is measured at low power level
            strDis1 = "Cal Unit" + Str(bytCoupler) + " FWD Lo"
            strDis2 = "Ant out " + "50" + Chr(244) + " ->OK"
            Call LogAmpCal(1 , 7 , 11)            '1=FWD, 7=CaldBmLo, 11=CalFWDADCLo

         Case 33                                  'Calibrate REV low power level
            'Coupler in reverse operation: Tx @ Ant out, Dummy load @ TX in
            'REV port 2 is measured at low power level
            strDis1 = "Cal Unit" + Str(bytCoupler) + " REV Lo"
            strDis2 = "TRX in " + "50" + Chr(244) + "  ->OK"
            Call LogAmpCal(2 , 7 , 13)            '2=REV, 7=CaldBmLo, 14=CalREVADCLo

         Case 34                                  'Calibrate FWD high power level
            'Coupler in normal operation: Tx @ TX in, Dummy load @ Ant out
            'FWD port 1 is measured at high power level
            strDis1 = "Cal Unit" + Str(bytCoupler) + " FWD Hi"
            strDis2 = "Ant out " + "50" + Chr(244) + " ->OK"
            Call LogAmpCal(1 , 9 , 15)            '1=FWD, 9=CaldBmHi, 15=CalFWDADCHi

         Case 35                                  'Calibrate REV high power level
            'Coupler in reverse operation: Tx @ Ant out, Dummy load @ TX in
            'REV port 2 is measured at high power level
            strDis1 = "Cal Unit" + Str(bytCoupler) + " REV Hi"
            strDis2 = "TRX in " + "50" + Chr(244) + "  ->OK"
            Call LogAmpCal(2 , 9 , 17)            '2=REV, 9=CaldBmHi, 17=CalREVADCHi

         Case 36                                  'Setup finished
            Call ClearLCDLine(1 , bytChar)
            LCD "Setup finished"
            If CouplerDataComplete(bytCoupler) = 1 Then       'Coupler data complete
               wrdAddr = wrdCouplerAddr
               bytCouplerData = 1                 'Coupler data complete
               WriteEEPROM bytCouplerData , wrdAddr
               Call ClearLCDLine(2 , bytChar)
               LCD "Unit " ; Str(bytCoupler) ; " complete"
               bitSetup = 0                       'Enter normal mode
            Else
               Call ClearLCDLine(2 , bytChar)
               LCD "* Data missing *"
               bitSetup = 1                       'Enter setup
            End If
            Wait 2
            Goto Restart

      End Select

   Loop

End If

End

'==============================================================================
Timer0_isr:
'Interrupt Service Routine for Timer0 (Keys)
'got from Thomas Baer in microcontroller.net
'Original (ASM and C) by Peter Dannegger
'http://www.mikrocontroller.net/topic/tasten-entprellen-bulletproof
'Input:  Key_port       port connected to the keys
'        bytKey_mask    Bit pattern to mask the keys enabled for short/long
'                       Key number (0...7) to be checked for short/long
'                       e.g. "00000101" Keys 0 and 2 are short/long keys
'Output: bytKey_press   bit pattern, key which changed from open to pressed
'                       Reset after evaluation in the main program
'                       Bits remain = 1 until resetted in the main program
'        bytKey_state   bit pattern, Key state, 1:pressed or 0:not pressed
'                       Bits = 1 while key is pressed, = 0: Key is released
'        bytKey_save    last state of bytKey_state before entering Timer0_isr
'                       Bits 0...7: keys 0...7 @ Key_port

   Timer0 = Presettimer0
   bytKey_save = bytKey_state                     'save old status before keys were pressed
   bytIwr0 = Key_port                             'get Bits of the KeyPort
   bytIwr0 = Not bytIwr0
   bytIwr0 = bytIwr0 Xor bytKey_state
   bytKey_ct0 = bytKey_ct0 And bytIwr0
   bytKey_ct1 = bytKey_ct1 And bytIwr0
   bytKey_ct0 = Not bytKey_ct0
   bytKey_ct1 = bytKey_ct1 Xor bytKey_ct0
   bytIwr0 = bytIwr0 And bytKey_ct0
   bytIwr0 = bytIwr0 And bytKey_ct1
   bytKey_state = bytKey_state Xor bytIwr0
   bytIwr0 = bytIwr0 And bytKey_state
   bytKey_press = bytKey_press Or bytIwr0

   'Key long press or auto repeat
   bytIwr0 = bytKey_state AND bytKey_mask         'Mask out short/long keys
   If bytIwr0 > 0 Then                            'Short/long key still pressed
      Incr bytTimer0_keys                         'Key press counter
   End If

   Incr wrdTimer0_PEP                             'SSB PEP counter

Return

'=========================================================================Keys=
Function KeyPressedLong(byVal bytKey_Long As Byte) As Byte
   'Check if key bytKey_Long (0...7) in Key_port is pressed short or long
   'Output: KeyPressedLong =  0: evaluating, key is still pressed
   '                          1: key is released, pressed short
   '                          2: key is released, pressed long

   If bytKey_state.bytKey_Long = 1 Then           'Key still pressed
      KeyPressedLong = 0
   Else                                           'Key released
      If bytTimer0_keys < bytTimer0_long Then     'short time elapsed
         KeyPressedLong = 1                       'Pressed short
      Else
         KeyPressedLong = 2                       'Pressed long
      End If
      bytTimer0_keys = 0
      bytKey_press.bytKey_Long = 0                'Reset pressed flag
   End If
End Function

'=========================================================================Keys=
Function KeyAutoRepeat(byVal bytKey_Rep As Byte) As Byte
'Check if key bytKey_Rep (0...7) in Key_port is pressed long for auto repeat
'Key status bytKey_state.bytKey_Rep is returned from Timer0_isr
'Input:  bytKey_state.bytKey_Rep =0: Key #bytKey_Rep is not   pressed
'                                =1: Key #bytKey_Rep is still pressed
'        bytTimer0_keys          Counter returned from Timer0_isr
'                                Incremented per 12.8 msec
'                                For different Timer0 timing check conditions
'                                "bytTimer0_keys >/< 20"
'Output: KeyAutoRepeat           =0: key is not pressed
'                                =1: key is pressed short, no auto repeat
'                                =2: key is pressed long,     auto repeat

   KeyAutoRepeat = 0
   If bytKey_state.bytKey_Rep = 1 Then            'Key still pressed
      If bytTimer0_keys > 20 Then
         KeyAutoRepeat = 2                        'Pressed long, auto repeat
         bytTimer0_keys = 0                       'Reset Timer0 counter
         Waitms 100                               'Step speed
      End If
   Else                                           'Key released
      If bytTimer0_keys < 20 Then                 '~ 250 msec
         KeyAutoRepeat = 1                        'Pressed short, no auto repeat
         bytTimer0_keys = 0                       'Reset Timer0 counter
         bytKey_press.bytKey_Rep = 0              'Reset pressed flag
      End If
   End If
End Function

'==============================================================================
ADC_isr:
'Interrupt Service Routine for ADC, conversion complete
'Read 10bit ADC Data Registers ADCL & ADCH
'bytADCLo and bytADCHi are overlayed with wrdADC
'Set next MUX channel and start ADC again
'Input:  bytADChannel   Current ADC channel
'        bytADCUsed     Max ADC channels - 1
'Output: wrdADCVal(i)   ADC value read for channel i
'        bitADCdone     =1: ADC accumulation complete

   bytADCLo = ADCL                                'Read ADCL first...
   bytADCHi = ADCH                                '... then ADCH
   bytADCValIdx = bytADChannel + 1                'Next value array position
   wrdADCVal(bytADCValIdx) = wrdADC               '10bit ADC value
   Incr bytADChannel                              'Next MUX channel
   If bytADChannel > bytADCUsed Then              'All ADC channels read
      bytADChannel = 0                            'Start with channel 0 again
      bitADCdone = 1                              'Accumulation complete
   End If
   Admux = bytADMUX Or bytADChannel               'Set next MUX channel
   ADCSRA.ADSC = 1                                'Start ADC again
Return

'==============================================================================
Sub GetADCData
'Get accumulated ADC Data
'Input:  bitADCdone     =1: ADC conversion complete (from ADC_isr)
'        wrdADCVal(1)   Current ADC counts forward
'        wrdADCVal(2)   Current ADC counts reverse
'        bytADCRep      Number of repetitive ADC accumulations
'Output: bitADCMean     =1: new ADC mean counts available
'        wrdADCMean(1   ADC mean counts forward
'        wrdADCMean(2)  ADC mean counts reverse
'        wrdADCPEP      ADC max counts forward (PEP)
'        bytADCRun      Current ADC accumulation number
'        bitADCdone     =0 if ADC data were read and summed

   bytADCOvl = 0                                  'Check for ADC overflow
   If wrdADCVal(1) > 1022 Then bytADCOvl = 1
   If wrdADCVal(2) > 1022 Then bytADCOvl = 2

   'Get max ADC count for PEP in FWD channel 1
   If wrdTimer0_PEP < wrdTimer0_PEPhold Then      'Current PEP evaluation period
      If wrdADCVal(1) > wrdADCPEP Then            'Index 1 = FWD power
         wrdADCPEP = wrdADCVal(1)                 'New max ADC PEP value
      End If
   Else
      wrdPEP = wrdADCPEP                          'Accumulated PEP
      wrdTimer0_PEP = 0                           'Next PEP evaluation period
      wrdADCPEP = 0
   End If

   'Get mean values for SWR and avg. power
   If bytADCRun < bytADCRep Then                  'Accumulate bytADCRep times
      wrdADCMean(1) = wrdADCMean(1) + wrdADCVal(1)       'ADC sum forward
      wrdADCMean(2) = wrdADCMean(2) + wrdADCVal(2)       'ADC sum reverse
      Incr bytADCRun
      bitADCMean = 0                              'ADC accumulating
   Else                                           'Mean values complete
      sngTmp0 = wrdADCMean(1)
      sngTmp0 = sngTmp0 / sngADCRep
      wrdADCMean(1) = sngTmp0                     'ADC mean forward
      sngTmp0 = wrdADCMean(2)
      sngTmp0 = sngTmp0 / sngADCRep
      wrdADCMean(2) = sngTmp0                     'ADC mean reverse
      bitADCMean = 1                              'ADC mean value available
   End If
   bitADCdone = 0

End Sub

'==============================================================================
Sub TwoPointCal(byVal bytOpt As Byte)
'Calculate the linear equation y=mx+b from the 2 calibration points
'm = (y2-y1)/(x2-x1), b=(y1*x2-y2*x1)/(x2-x1) calculated for FWD and REV
'Input:  bytOpt               =1: Forward, =2: Reverse
'        intCaldBmLo    = y1  Low  dBm calibration point(dBm x 10)
'        intCaldBmHi    = y2  High dBm calibration point (dBm x 10)
'        Forward calibration
'        wrdCalFwdADCLo = x1  Forward ADC counts at low  cal. point
'        wrdCalFwdADCHi = x2  Forward ADC counts at high cal. point
'        Reverse calibration
'        wrdCalRevADCLo = x1  Reverse ADC counts at low  cal. point
'        wrdCalRevADCHi = x2  Reverse ADC counts at high cal. point
'Output: sngFwdGrd            Gradient m forward
'        sngFwdOrd            Ordinate b forward
'        sngRevGrd            Gradient m reverse
'        sngRevOrd            Ordinate b reverse

   Select Case bytOpt
      Case 1                                      'Forward
         wrdTmp0 = wrdCalFwdADCLo                 'x1
         wrdTmp1 = wrdCalFwdADCHi                 'x2
      Case Else                                   'Reverse
         wrdTmp0 = wrdCalRevADCLo                 'x1
         wrdTmp1 = wrdCalRevADCHi                 'x2
   End Select
   intTmp0 = intCaldBmLo                          'y1 (dBm x 10)
   intTmp1 = intCaldBmHi                          'y2 (dBm x 10)

   'Calculate gradient m forward or reverse, m = (y2-y1)/(x2-x1)
   intTmp2 = intTmp1 - intTmp0                    'y2-y1 (dBm x 10)
   wrdTmp2 = wrdTmp1 - wrdTmp0                    'x2-x1
   sngTmp0 = intTmp2                              'y2-y1 (dBm x 10)
   sngTmp0 = 0.1 * sngTmp0                        'y2-y1 (dBm)
   sngTmp1 = wrdTmp2                              'x2-x1
   sngTmp2 = sngTmp0 / sngTmp1                    '[y2-y1]/[x2-x1]=gradient m
   Select Case bytOpt
      Case 1                                      'Forward
         sngFwdGrd = sngTmp2
      Case Else                                   'Reverse
         sngRevGrd = sngTmp2
   End Select

   'Calculate ordinate b forward or reverse, b=(y1*x2-y2*x1)/(x2-x1)
   sngTmp0 = intTmp0                              'y1 (dBm x 10)
   sngTmp0 = 0.1 * sngTmp0                        'y1 (dBm)
   sngTmp1 = wrdTmp1                              'x2
   sngTmp0 = sngTmp0 * sngTmp1                    'y1*x2
   sngTmp1 = intTmp1                              'y2 (dBm x 10)
   sngTmp1 = 0.1 * sngTmp1                        'y2 (dBm)
   sngTmp2 = wrdTmp0                              'x1
   sngTmp1 = sngTmp1 * sngTmp2                    'y2*x1
   sngTmp0 = sngTmp0 - sngTmp1                    'y1*x2-y2*x1
   sngTmp1 = wrdTmp2                              'x2-x1
   sngTmp2 = sngTmp0 / sngTmp1                    '[y1*x2-y2*x1]/[x2-x1]=ordinate b
   Select Case bytOpt
      Case 1                                      'Forward
         sngFwdOrd = sngTmp2
      Case Else                                   'Reverse
         sngRevOrd = sngTmp2
   End Select

End Sub

'==============================================================================
Sub ShowPower(byVal bytRowPwr As Byte)
'Calculate an show forward & reverse RF Power
'Input:  bytRowPwr      LCD row
'        wrdADCPEP      ADC max peak for PEP calculation
'        wrdADCMean(1)  ADC mean forward for SWR or avg. power
'        wrdADCMean(2)  ADC mean reverse for SWR
'        sngFwdGrd      2 point calibration line gradient forward
'        sngFwdOrd      2 point calibration line ordinate forward
'        bitShowAvg     =0: peak power (PEP), =1: average power
'Output: bytPWRerr      Error flag
'                       = 0: no error
'                       = 1: forward power above upper detection limit
'                       = 2: forward power ADC overflow
'                       = 3: reverse power below lower detection limit
'                       = 4: reverse power above upper detection limit
'                       = 5: reverse power ADC overflow
'                       = 6: reverse > forward
'                       =16: No checks performed except overflow
'        bitTXoff       = 1: TX is off (FWD power < lower detection limit)

   If bitShowAvg = 1 Then                         'avg. power
      sngTmp0 = wrdADCMean(1)
      Waitms 100                                  'Slow down flickering
   Else                                           'PEP
      sngTmp0 = wrdPEP
   End If
   bytPWRerr = 16

   'Forward power -------------------------------------------------------------
   'Calculate forward power from 2 point calibration line
   'Add the total coupler attenuation to get the real TX power
   sngFwddBm = sngFwdGrd * sngTmp0
   sngFwddBm = sngFwddBm + sngFwdOrd              'Coupled forward power (dBm)
   bitTXOFF = 0                                   'TX is on

   If bytADCOvl = 1 Then                          'ADC overflow FWD
      strTmp16 = "F Ovfl "
      Call DisplayString(7)
      bytPWRerr = 2
   Else
      Select Case sngFwddBm
         Case Is < sngMinFWDdBm                   'FWD power < lower FWD detection limit
            bitTXOFF = 1                          'Set TX off flag
            strTmp16 = "F   0.0"
            Call DisplayString(7)
         Case Is > sngMaxdBm                      'FWD power > upper detection limit
            strTmp16 = "F >>>>>"
            Call DisplayString(7)
            bytPWRerr = 1
         Case Else
            sngTmp1 = sngFwddBm + sngCoupAtt      'Forward TX power (dBm)
            sngFwdPwr = dBm2Watt(sngTmp1)         'Forward TX power (W)
            wrdTmp0 = sngFwdPwr * 10              'Forward TX power, show 1 decimal
            strTmp16 = Str(wrdTmp0)
            strTmp16 = Format(strTmp16 , "  0.0")
            strTmp16 = "F " + strTmp16
            Call DisplayString(7)
            bytPWRerr = bytPWRerr - 8             '=8, next check for reverse
      End Select
   End If

   'Reverse Power -------------------------------------------------------------
   'Calculate reverse power from 2 point calibration line
   'Add the total coupler attenuation to get the real reflected power
   sngTmp0 = wrdADCMean(2)
   sngRevdBm = sngRevGrd * sngTmp0
   sngRevdBm = sngRevdBm + sngRevOrd              'Reverse power (dBm)

   If bytADCOvl = 2 Then                          'ADC overflow REV
      strTmp16 = "R Ovfl "
      Call DisplayString(16)
      bytPWRerr = 5
   Else
      If bitTXOFF = 1 Then                        'TX is off
         strTmp16 = " R  0.00W"                   'REV not measured
         bytPWRerr = 0
         Call DisplayString(16)
      Else                                        'TX is on
         Select case sngRevdBm
            Case Is < sngMinREVdBm                'REV < lower detection limit
               strTmp16 = "R <<<<< "
               bytPWRerr = 3
               Call DisplayString(16)
            Case Is > sngMaxdBm                   'REV > upper detection limit
               strTmp16 = "R >>>>> "
               Call DisplayString(16)
               bytPWRerr = 4
            Case Else
               sngTmp1 = sngRevdBm + sngCoupAtt   'Reverse TX power (dBm)
               sngRevPwr = dBm2Watt(sngTmp1)      'Reverse TX power (W)
               wrdTmp0 = sngRevPwr * 100          'Reverse TX power, show 2 decimals
               strTmp16 = Str(wrdTmp0)
               strTmp16 = Format(strTmp16 , " 0.00")
               strTmp16 = " R " + strTmp16 + "W"
               Call DisplayString(16)
               bytPWRerr = bytPWRerr - 8          '=0 if FWD power was OK
         End Select
      End If
   End If
   Locate bytRowPwr , 1
   LCD strDis1                                    'Show me

 End Sub

'======================================================================Display=
Sub DisplayString(byVal bytPosLast As Byte)
'Store temporary string  bytTmp16 to display string bytDis
'from position bytPosLast backwards
'Input:  bytTmp16  (=strTmp16 by overlay)
'        bytPos   right position in bytDis (=strDis by overlay)

   bytTmp0 = Len(strTmp16)
   If bytTmp0 <= bytPosLast Then                  'Enough room
      bytTmp1 = bytPosLast + 1
      For bytTmp2 = bytTmp0 To 1 Step -1          'From right to left
          bytTmp1 = bytTmp1 - 1
          bytDis1(bytTmp1) = bytTmp16(bytTmp2)
      Next bytTmp2
   Else                                           'No room, overflow
      For bytTmp2 = 1 To bytPosLast
          bytDis1(bytTmp2) = "#"
      Next bytTmp2
   End If
End Sub

'==============================================================================
Function dBm2Watt(byVal sngdBm As Single) As Single
'Convert dbm to Watt, W = [10^(dBm/10)]/1000
'Input:  sngdBm     dBm

   sngTmp0 = 10
   sngTmp1 = 0.1 * sngdBm
   dBm2Watt = Power(sngTmp0 , sngTmp1)            'mW
   dBm2Watt = 0.001 * dBm2Watt                    'W

End Function

'==============================================================================
Sub ShowSWR(byVal bytRowSWR As Byte)
'Paint value sngVal on LCD, Row bytRow, as bar graph
'Input:  bytRowSWR      LCD row
'        sngFwdPwr      Forward power (W)
'        sngRevPwr      Reverse power (W)
'        sngSWRMax      max. SWR to be displayed (end of bar)
'        sngSWRBarCal   calibration factor (dots / max value)

   If bitTXOFF = 1 Then                           'TX is off
      Locate bytRowSWR , 1
      LCD "TX is off       "
      LED_SWR = 0                                 'LED_SWR off
      Exit Sub                                    'SWR not displayed
   End If

   sngTmp0 = sngRevPwr / sngFwdPwr                'Ratio REV/FWD
   sngTmp1 = SQR(sngTmp0)                         'rho
   If sngTmp1 < 1 Then
      sngSWR = 1 + sngTmp1
      sngTmp0 = 1 - sngTmp1
      sngSWR = sngSWR / sngTmp0                   'SWR
   Else
      sngSWR = 100                                'SWR undefinded
   End If

   If sngSWR > 9.99 Then
      sngSWR = 9.99                               'SWR display limit
   End If
   If sngSWR > sngSWRMax Then                     'Max SWR exeeded?
      LED_SWR = 1                                 'Yes, LED_SWR on
   Else
      LED_SWR = 0                                 'No, LED_SWR off
   End If

   If sngSWR > sngSWRBarMax Then                  'Suppress bargraph for SWR = 1
      sngTmp0 = sngSWRBarMax - 1                  'Display limit
   Else
      sngTmp0 = sngSWR - 1                        'Current SWR
   End If

   sngTmp0 = sngSWRBarCal * sngTmp0               'Bar calibration
   wrdTmp2 = sngTmp0                              'Number of bar dots
   wrdTmp0 = wrdTmp2 / 5                          'Number of 5 dot full blocks
   wrdTmp1 = wrdTmp2 Mod 5                        'Remaining partial single bars

   strDis2 = ""
   If wrdTmp0 > 0 Then                            'Full 5 dot blocks
      For bytTmp0 = 1 To wrdTmp0
         strDis2 = strDis2 + Chr(5)
      Next bytTmp0
   End If
   If wrdTmp1 > 0 Then
      strDis2 = strDis2 + Chr(wrdTmp1)            'Last bar(s) < 5 dots
   End If

   wrdTmp1 = bytBarLen - 4                        'Space for Tag 'SWR'?
   If wrdTmp0 < wrdTmp1 Then
      bytTmp0 = 1                                 'yes
   Else
      bytTmp0 = 0                                 'no
   End If

   wrdTmp0 = Len(strDis2)                         'string length up to now
   wrdTmp1 = bytChar - wrdTmp0                    'remaining characters
   wrdTmp1 = wrdTmp1 - 4                          '4 char's for value
   If wrdTmp1 > 0 Then
      strDis2 = strDis2 + Space(wrdTmp1)          'Fill with spaces
   End If
   If bytTmp0 = 1 Then                            'Add tag 'SWR'
      wrdTmp0 = Len(strDis2)
      wrdTmp1 = wrdTmp0 - 4
      strDis2 = Left(strDis2 , wrdTmp1)
      strDis2 = strDis2 + "SWR "
   End If

   wrdTmp0 = sngSWR * 100                         'show 2 decimals
   strTmp16 = str(wrdTmp0)
   strTmp16 = Format(strTmp16 , "0.00")
   strDis2 = strDis2 + strTmp16                   'Add sngSWR value at the end
   Locate bytRowSWR , 1
   LCD strDis2                                    'Show me

End Sub

'==============================================================================
Sub SetValue(byVal bytRow As Byte , byVal bytType As Byte , byVal wrdScale As Word)
'Set EEPROM value via keys and restore it to EEPROM
'EEPROM data are byte, Word or Integer, perhaps scaled with factors 10, 100 or 1000
'e.g. 1.234V ist stored as 1234mV in EEPROM
'Values are changed via up and down key
'Input:  bytRow                  LCD row to display value
'        bytType                 Type of value
'                                1 = Byte
'                                2 = Word
'                                3 = Integer
'        bytData                 Input variable for bytType = 1 (Byte)
'        wrdData                 Input variable for bytType = 2 (Word)
'        intData                 Input variable for bytType = 3 (Integer)
'        wrdScale                Scale factor of EEPROM value, 1, 10, 100 or 1000
'                                EEPROM value = real value * wrdScale
'                                e.g. Word variable (mV) to be shown as Volt:
'                                wrdScale=1000, 1234 mV divided by 1000, shown as 1.234V
'                                stored as 1.234 * 1000 (mV, Word)
'        bytCursorFirst          First LCD column to show data
'        wrdAddr                 EEPROM address to store value

   Select Case bytType
      Case 1                                      'Input byte variable bytData
         strTmp16 = Str(bytData)
      Case 2                                      'Input Word variable wrdData
         strTmp16 = Str(wrdData)
      Case Else                                   'Input Integer variable intData
         strTmp16 = Str(intData)
   End Select

   Select Case wrdScale
      Case 1
         strTmp16 = Format(strTmp16 , " 0")
      Case 10
         strTmp16 = Format(strTmp16 , " 0.0")
      Case 100
         strTmp16 = Format(strTmp16 , " 0.00")
      Case 1000
         strTmp16 = Format(strTmp16 , " 0.000")
   End Select
   strTmp16 = strTmp16 + strUnit
   Locate bytRow , bytCursorFirst
   LCD strTmp16

   'Up key  pressed short: increment by bytDataInc
   '        pressed long : with autorepeat
   If bytkey_Up > 0 Then                          'Up short or long/autorepeat
      Select Case bytType
         Case 1
            bytData = bytData + bytDataInc
            If bytData > bytDataMax Then bytData = bytDataMax
            If bytMenu = 21 Then                  'LCD backlight PWM
               bytLCDPWM = bytData
               OCR1A = bytData                    'Set PWM value on OC1A (PB.1)
            End If
         Case 2
            wrdData = wrdData + bytDataInc
            If wrdData > wrdDataMax Then wrdData = wrdDataMax
         Case Else
            intData = intData + bytDataInc
            If intData > intDataMax Then intData = intDataMax
      End Select

   End If

   'Both Up and OK key pressed: return to previous menu
   If bytkey_Down > 0 And bytKey_OK > 0 Then
      If bytMenu > 21 Then decr bytMenu
      bytSetup = 0
      Exit Sub
   End If

   'Down key   pressed short: decrement by 1
   '           pressed long : with autorepeat
   If bytkey_Down > 0 Then                        'Down short or long/autorepeat
      Select Case bytType
         'bytDataDec
         Case 1
            If bytData >= bytDataDec Then
               bytData = bytData - bytDataDec
               If bytData < bytDataMin Then bytData = bytDataMin
            End If
            If bytMenu = 21 Then                  'LCD backlight PWM
               bytLCDPWM = bytData
               OCR1A = bytData                    'Set PWM value on OC1A (PB.1)
            End If
         Case 2
            If wrdData >= bytDataDec Then
               wrdData = wrdData - bytDataDec
               If wrdData < wrdDataMin Then wrdData = wrdDataMin
            End If
         Case Else
            intData = intData - bytDataDec
            If intData < intDataMin Then intData = intDataMin
      End Select

   End If

   'OK key pressed short: next menu, data are not stored
   '        pressed long : store current data to EEPROM, next menu
   Select Case bytKey_OK
      Case 1                                      'OK short, finish
         Incr bytMenu                             'Next menu
         bytSetup = 0
      Case 2                                      'OK long, store & finish
         Select Case bytType
            Case 1                                'Input byte variable bytData
               WriteEEPROM bytData , wrdAddr
            Case 2                                'Input Word variable wrdData
               Call EEPROMWriteWrd(wrdData)
            Case Else                             'Input Integer variable intData
               Call EEPROMWriteInt(intData)
         End Select
         Locate 2 , 16
         LCD "S"
         Incr bytMenu                             'Next menu
         bytSetup = 0
         Wait 1
   End Select

End Sub

'==============================================================================
Sub LogAmpCal(byVal bytChannel As Byte , byVal bytAddrdBm As Byte , byVal bytAddrADC As Byte)
'Calibrate Log Amplifier
'Input:  bytChannel     =1: forward, =2: reverse
'        wrdCouplerAddr Coupler EEPROM start address
'        bytAddrdBm     EEPROM address adder for coupler dBm value
'        bytAddrADC     EEPROM address adder for coupler ADC value
'        bytSetup       =0: Prepare data, =1: wait for OK, =2: calibrate
'        bytCoupler     Coupler number 1...4
'        strDis1        Display string row 1
'        strDis2        Display string row 2

Select Case bytSetup

   Case 0                                         'Prepare data
      bytSetup = 1
      Locate 1 , 1
      LCD strDis1
      wrdAddr = wrdCouplerAddr + 1                'Address coupling factor
      Call EEPROMReadWrd(wrdCoupFac)              'Coupling factor (dB x 10)
      wrdAddr = wrdCouplerAddr + 3                'Address log amp attenuator
      Call EEPROMReadWrd(wrdAmpAtt)               'Log amp attenuator (dB x 10)
      wrdAddr = wrdCouplerAddr + bytAddrdBm       'Address log amp cal dBm value
      Call EEPROMReadInt(intCalDBmLo)             'Log amp cal dBm value (dBm x 10)
      intTmp0 = intCalDBmLo + wrdCoupFac          'Log amp inp + coupling fac (dBm x 10)
      intTmp0 = intTmp0 + wrdAmpAtt               '... + attenuator = TX input (dBm x 10)
      strTmp16 = Str(intTmp0)
      strTmp16 = Format(strTmp16 , " 0.0")        'Show 1 decimal = dBm
      Locate 2 , 1
      LCD strDis2

   Case 1                                         'Wait for OK
      If bytKey_OK > 0 Then
         bytSetup = 2
         Call ClearLCDLine(2 , bytChar)
         LCD strTmp16 ; "dBm"
         bytCursorFirst = 10
         wrdAddr = wrdCouplerAddr + bytAddrADC    'Address ADC value
         bytADCRun = 0
         bytADChannel = 0                         'First ADC channel ADC0
         bitADCdone = 0
         wrdADCMean(1) = 0                        'ADC mean counts forward
         wrdADCMean(2) = 0                        'ADC mean counts reverse
         ADCSRA.ADSC = 1                          'Start ADC
      End If

   Case Else                                      'Measure ADC counts
      If bitADCdone = 1 Then                      'ADC accumulation complete
         Call GetADCData                          'Handle ADC values
         If bitADCMean = 1 Then                   'ADC mean data available
            wrdTmp1 = wrdADCMean(bytChannel)      '1=forward, 2=reverse
            strTmp16 = str(wrdTmp1)
            strTmp16 = Format(strTmp16 , "   0")
            Locate 2 , bytCursorFirst
            LCD strTmp16
            Waitms 100                            'Slow down flickering
            bytADCRun = 0                         'Initialize next ADC run
            wrdADCMean(1) = 0                     'Initialize for next ADC run
            wrdADCMean(2) = 0
         End If
      End If
      If bytKey_OK > 0 Then                       'OK pressed
         ADCSRA.ADSC = 0                          'Stop ADC
         Incr bytMenu                             'Next menu
         bytSetup = 0
         If bytKey_OK = 2 Then                    'OK pressed long: store to EEPROM
            wrdAddr = wrdCouplerAddr + bytAddrADC 'EEPROM address
            Call EEPROMWriteWrd(wrdTmp1)
            Locate 2 , 16
            LCD "S"
            Wait 1
         End If
      End If

End Select

End Sub

'==============================================================================
Sub ClearLCDLine(byVal bytRow As Byte , byVal bytChars As Byte)
'Clear LCD bytChars characters in line bytRow
   Locate bytRow , 1
   LCD SPC(bytChars)
   Locate bytRow , 1
End Sub

'==============================================================================
Sub CheckEEPROM
'Read coupler data from EEPROM or initialize data
'In a new controller or erased EEPROM data = 255
'If coupler data are stored, at least 1 coupler, byte # 1 = 1
'SWR bridge data:
'Byte Variable          Type     Description
' 0   --                Byte     Not used
' 1   --                Byte     = 1: data are stored
' 2   bytLCDPWM         Byte     PWM value for LCD backlight (0...255)
' 3   bytADCRep         Byte     Repetitive ADC runs for mean value
'Following blocks of data of 4 different couplers (units)
'Couplers may fit different frequencies (HF/VHF/UHF) or power (QRP/QRO)
'Blocks start with address 10, 40, 70, 100 (increment 30)
'First block of coupler data
'10   bytCouplerData    Byte     =0: initialized, =1: calibrated
'11   wrdCoupFac        Word     Coupling factor (dB x 10)
'13   wrdAmpAtt         Word     Log amp attenuator (dB x 10)
'15   wrdCalFreq        Word     Calibration frequency (MHz x 10)
'17   intCaldBmLo       Integer  Low  log amp dBm calibration point (dBm x 10)
'19   intCaldBmHi       Integer  High log amp dBm calibration point (dBm x 10)
'21   wrdCalFwdADCLo    Word     Forward ADC counts at low  calibration point
'23   wrdCalRevADCLo    Word     Reverse ADC counts at low  calibration point
'25   wrdCalFwdADCHi    Word     Forward ADC counts at high calibration point
'27   wrdCalRevADCHi    Word     Reverse ADC counts at high calibration point
'29   intMindBm         Integer  Lower detection limit (dBm x 10)
'31   intMaxdBm         Integer  Upper detection limit (dBm x 10)
'33   bytPEPhold        Byte     PEP peak sample & hold time (sec x 10)

   ReadEeprom bytTmp1 , 1                         'Check Address #1
   Locate 1 , 1
   If bytTmp1 = 1 Then                            'Read available EEPROM data
      LCD "Read EEPROM...  "
      'Read EEPROM header data ------------------------------------------------
      ReadEEPROM bytLCDPWM , 2                    'PWM value for LCD backlight
      ReadEEPROM bytADCRep , 3                    'Repetitive ADC runs for mean value

      'Read data for coupler bytCoupler ---------------------------------------
      wrdAddr = bytCoupler - 1
      wrdAddr = wrdAddr * 30
      wrdAddr = wrdAddr + 10
      ReadEEPROM bytCouplerData , wrdAddr
      wrdAddr = wrdAddr + 1
      Call EEPROMReadWrd(wrdCoupFac)              'Coupling factor (dB x 10)
      Call EEPROMReadWrd(wrdAmpAtt)               'Log amp attenuator (dB x 10)
      Call EEPROMReadWrd(wrdCalFreq)              'Coupler calibration frequency
      Call EEPROMReadInt(intCaldBmLo)             'Low  dBm calibration point (dBm x 10)
      Call EEPROMReadInt(intCaldBmHi)             'High dBm calibration point (dBm x 10)
      Call EEPROMReadWrd(wrdCalFwdADCLo)          'FWD ADC counts at low  calibration point
      Call EEPROMReadWrd(wrdCalRevADCLo)          'REV ADC counts at low  calibration point
      Call EEPROMReadWrd(wrdCalFwdADCHi)          'FWD ADC counts at high calibration point
      Call EEPROMReadWrd(wrdCalRevADCHi)          'REV ADC counts at high calibration point
      Call EEPROMReadInt(intMindBm)               'Lower detection limit (dBm x 10)
      Call EEPROMReadInt(intMaxdBm)               'Upper detection limit (dBm x 10)
      ReadEEPROM bytPEPhold , wrdAddr             'PEP peak sample & hold time (sec)
      Locate 2 , 1
      If bytCouplerData = 1 Then                  'Coupler data calibrated
         LCD "Unit" ; bytCoupler ; " calibrated"
         If bitSetup = 0 Then
            bitSetup = 0                          'No setup necessary
         Else                                     'Setup forced on power up
            bitSetup = 1
         End If
      Else                                        'Coupler data not calibrated
         LCD "Unit" ; bytCoupler ; " not Calib."
         bitSetup = 1                             'Setup necessary
      End If
      Wait 2

   Else                                           'EEPROM is empty, initialize data

      LCD "Init EEPROM...  "
      'Initialize EEPROM header data ------------------------------------------
      bytLCDPWM = 100
      WriteEEPROM bytLCDPWM , 2                   'PWM value for LCD backlight
      bytADCRep = 20
      WriteEEPROM bytADCRep , 3                   'Repetitive ADC runs for mean value

      'Initialize coupler data ------------------------------------------------
      For bytTmp1 = 1 To 4
         wrdAddr = bytTmp1 - 1
         wrdAddr = wrdAddr * 30
         wrdAddr = wrdAddr + 10
         bytCouplerData = 0                       'Coupler data initialized
         WriteEEPROM bytCouplerData , wrdAddr
         wrdAddr = wrdAddr + 1
         wrdCoupFac = 250                         '25dB
         Call EEPROMWriteWrd(wrdCoupFac)          'Coupling factor (dB x 10)
         wrdAmpAtt = 0
         Call EEPROMWriteWrd(wrdAmpAtt)           'Log amp attenuator (dB x 10)
         wrdCalFreq = 10                          '10MHz
         Call EEPROMWriteWrd(wrdCalFreq)          'Coupler calibration frequency (MHz)
         intCaldBmLo = -600                       '-60dBm
         Call EEPROMWriteInt(intCaldBmLo)         'Low  dBm calibration point (dBm x 10)
         intCaldBmHi = 100                        '+10dBm
         Call EEPROMWriteInt(intCaldBmHi)         'High dBm calibration point (dBm x 10)
         wrdCalFwdADCLo = 0
         Call EEPROMWriteWrd(wrdCalFwdADCLo)      'FWD ADC counts at low  calibration point
         wrdCalRevADCLo = 0
         Call EEPROMWriteWrd(wrdCalRevADCLo)      'REV ADC counts at low  calibration point
         wrdCalFwdADCHi = 0
         Call EEPROMWriteWrd(wrdCalFwdADCHi)      'FWD ADC counts at low  calibration point
         wrdCalRevADCHi = 0
         Call EEPROMWriteWrd(wrdCalRevADCHi)      'REV ADC counts at low  calibration point
         intMindBm = -700                         '-70dBm
         Call EEPROMWriteInt(intMindBm)           'Lower detection limit (dBm x 10)
         intMaxdBm = 150                          '+15dBm
         Call EEPROMWriteInt(intMaxdBm)           'Upper detection limit (dBm x 10)
         bytPEPhold = 10                          '1 sec
         WriteEEPROM bytPEPhold , wrdAddr         'PEP peak sample & hold time (sec)
     Next bytTmp1
     bytTmp1 = 1
     WriteEEPROM bytTmp1 , 1
     Locate 2 , 1
     LCD "... Init done   "
     bitSetup = 1                                 'Setup necessary
     Wait 2
   End If

End Sub

'==============================================================================
Function CouplerDataComplete(byVal bytCoupNum As Byte) As Byte
'Check if EEPROM data of Coupler bytCoupNum are complete
'Input:  bytCoupNum           Number of coupler (1...4)
'Output: 'CouplerDataComplete =1: complete, =0: else

   CouplerDataComplete = 0
   wrdAddr = bytCoupNum - 1
   wrdAddr = wrdAddr * 30
   wrdAddr = wrdAddr + 10                         'Coupler EEPROM Start address
   bytTmp1 = 0
   wrdAddr = wrdAddr + 1
   Call EEPROMReadWrd(wrdCoupFac)                 'Coupling factor (dB x 10)
   If wrdCoupFac > 0 Then bytTmp1 = bytTmp1 + 1
   Call EEPROMReadWrd(wrdAmpAtt)                  'Log amp attenuator (dB x 10)
   If wrdAmpAtt >= 0 Then bytTmp1 = bytTmp1 + 1
   Call EEPROMReadWrd(wrdCalFreq)                 'Coupler calibration frequency
   If wrdCalFreq > 0 Then bytTmp1 = bytTmp1 + 1
   Call EEPROMReadInt(intCaldBmLo)                'Low  dBm calibration point (dBm x 10)
   If intCaldBmLo <> 0 Then bytTmp1 = bytTmp1 + 1
   Call EEPROMReadInt(intCaldBmHi)                'High dBm calibration point (dBm x 10)
   If intCaldBmHi <> 0 Then bytTmp1 = bytTmp1 + 1
   Call EEPROMReadWrd(wrdCalFwdADCLo)             'FWD ADC counts at low  calibration point
   If wrdCalFwdADCLo > 0 Then bytTmp1 = bytTmp1 + 1
   Call EEPROMReadWrd(wrdCalRevADCLo)             'REV ADC counts at low  calibration point
   If wrdCalRevADCLo > 0 Then bytTmp1 = bytTmp1 + 1
   Call EEPROMReadWrd(wrdCalFwdADCHi)             'FWD ADC counts at high calibration point
   If wrdCalFwdADCHi > 0 Then bytTmp1 = bytTmp1 + 1
   Call EEPROMReadWrd(wrdCalRevADCHi)             'REV ADC counts at high calibration point
   If wrdCalRevADCHi > 0 Then bytTmp1 = bytTmp1 + 1
   Call EEPROMReadInt(intMindBm)                  'Lower detection limit (dBm x 10)
   If intMindBm <> 0 Then bytTmp1 = bytTmp1 + 1
   Call EEPROMReadInt(intMaxdBm)                  'Upper detection limit (dBm x 10)
   If intMaxdBm <> 0 Then bytTmp1 = bytTmp1 + 1
   ReadEEPROM bytPEPhold , wrdAddr                'PEP peak sample & hold time (sec)
   If bytPEPhold >= 0 Then bytTmp1 = bytTmp1 + 1
   If bytTmp1 = 12 Then
      CouplerDataComplete = 1
   End If
End Function

'==============================================================================
Sub EEPROMWriteWrd(byRef wrdVal As Word)
   'Decode wrdWord to 2 bytes of bytWrdEEP for writing to EEPROM
   'Input:  wrdWord to be decoded & written to EEPROM
   '        wrdAddr first EEPROM address

   wrdEEPROM = wrdVal                             'wrdEEPROM overlayed @ bytWrdEEP
   For bytTmp0 = 1 To 2
      WriteEEPROM bytWrdEEP(bytTmp0) , wrdAddr
      Incr wrdAddr
   Next bytTmp0
End Sub

'==============================================================================
Sub EEPROMWriteInt(byRef intVal As Integer)
   'Decode intVal to 2 bytes of bytIntEEP for writing to EEPROM
   'Input:  intVal to be decoded & written to EEPROM
   '        wrdAddr first EEPROM address

   intEEPROM = intVal                             'intEEPROM overlayed @ bytIntEEP
   For bytTmp0 = 1 To 2
      WriteEEPROM bytIntEEP(bytTmp0) , wrdAddr
      Incr wrdAddr
   Next bytTmp0
End Sub

'==============================================================================
Sub EEPROMReadWrd(byRef wrdVal As Word)
   'Encode 2 bytes read from EEPROM to wrdWord
   'Input:  wrdAddr first EEPROM address
   'Output: wrdWord encoded unsigned integer (word) variable

   For bytTmp0 = 1 To 2
      ReadEEPROM bytWrdEEP(bytTmp0) , wrdAddr
      Incr wrdAddr
   Next bytTmp0
   wrdVal = wrdEEPROM                             'wrdEEPROM overlayed @ bytWrdEEP
End Sub

'==============================================================================
Sub EEPROMReadInt(byRef intVal As Integer)
   'Encode 2 bytes read from EEPROM to intVal
   'Input:  wrdAddr first EEPROM address
   'Output: intVal encoded signed integer variable

   For bytTmp0 = 1 To 2
      ReadEEPROM bytIntEEP(bytTmp0) , wrdAddr
      Incr wrdAddr
   Next bytTmp0
   intVal = intEEPROM                             'intEEPROM overlayed @ bytIntEEP
End Sub